<?php

namespace Importacao\Logic\Integracao\Mastermaq\Classes;


/**
 * Description of Migracao
 *
 * @author matheussilva
 */
abstract class Migracao {
    
    const COMPETENCIA = '01/01/2000';
    private $con;
    protected $file_name;
    protected $file_name_;
    protected $file_name_ent;
    public $objects = array();
    protected $columns = array();
            
    
    function __construct($path) {
        $this->connect($path);
    }

    /**
     * Conecta a um arquivo DBF.
     * 
     * @param string $path
     * @return bool
     * @throws \Exception
     */
    public function connect($path){
        try{
            $this->con = dbase_open($path, 0);
            $this->file_name_ = substr($path, strrpos($path, '\\')+1, 8);
            $this->file_name_ent = substr($this->file_name_, 5, 8);
            
            return(true);
            
        }  catch (\Exception $e){
            throw $e;
        }
    }

    /**
     * Retorna a conexão existente, se ela estiver aberta.
     * @throws \Exception
     */
    public function getConnection(){
        try {
            return $this->con;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Recupera as informações do arquivo e salva em um atributo.
     * @throws \Exception
     */
    public function getInfo(){
        try{
            
            $regs = array();
            $num = dbase_numrecords($this->con);
            
            if($num>1){
                for($c=1; $c<=$num; $c++){
                    $reg = dbase_get_record_with_names($this->con, $c);
                    $reg = array_map( 'utf8_encode', $reg);
                    $regs[] = array_map('trim',$reg);
                }
                
                $this->createInfo($regs);
            }
        }  catch (\Exception $e){
            throw $e;
        }
    }
    public function getInfo_(){
        try{
            
            $regs = array();
            $num = dbase_numrecords($this->con);
            
            if($num>1){
                for($c=1; $c<=$num; $c++){
                    $reg = dbase_get_record_with_names($this->con, $c);
                    $reg = array_map( 'utf8_encode', $reg);
                    $regs[] = array_map('trim',$reg);
                }
            }
            return $regs;
        }  catch (\Exception $e){
            throw $e;
        }
    }
    
    /**
     * Fecha a conexão com o arquivo.
     * @throws \Exception
     */
    public function close(){
        try{   
            dbase_close($this->con);
        }catch(\Exception $e){
            throw $e;
        }
        
    }
    /**
     * Grava o conteúdo em um arquivo em disco.
     * 
     * @param Object $content
     * @param string $exit_path
     * @throws \Exception
     */
    private function generateFile($content, $exit_path){
        try{
            $fp = fopen($exit_path, 'w+');
            fwrite($fp, $content);
            fclose($fp);
        }catch(\Exception $e){
            throw $e;
        }
    }
    
    /**
     * Gera o conteúdo em JSON e o grava em um arquivo.
     * 
     * @param type $exit_path
     * @throws \Exception
     */
    public function gerarJson($exit_path){
        try{
            if(!empty($this->objects)){
                $json = json_encode($this->objects);
                $this->generateFile($json, $exit_path.'\\'.$this->file_name.'.json');
            }else{
                throw new \Exception("Informa&ccedil;&atilde;o n&atilde;o gerada. Execute getInfo() para gerar a informa&ccedil;&atilde;o.");
            }
        }catch(\Exception $e){
            throw $e;
        }
    }
    
    /**
     * Gera o conteúdo XLS e o grava em um aquivo.
     */
    public function gerarXls($exit_path){

//        
        ini_set('memory_limit', '-1');
        set_time_limit(0);

        //Criar um objeto
        $xls = new \PHPExcel();
        
        //Criar as colunas
        $n = 'A';
        foreach ($this->columns as $column){
            $xls->setActiveSheetIndex(0)->setCellValue("{$n}1",$column );
            $n++;
        }
        
        /*
         * Seta os valores em cada célula.
         * $row = Linha de inserção
         * $col = Coluna de inserção
         * $reg é o dado que será inserido
         */
        $row = 2;
        foreach ($this->objects as $object){
//            $object = (array)$object;
//            $col = 0;
//            foreach ($object as $reg){
//                $xls->getActiveSheet()->setCellValueByColumnAndRow($col, $row, $reg);
//                $col++;
//            }
//            $row++;

            // Seta o estilo da célula para o tipo de texto '@'
            $object = (array)$object;
            $col = "A";
            foreach ($object as $reg){
//                $xls->getActiveSheet()->getStyle($col.$row)->getNumberFormat()->setFormatCode(\PHPExcel_Style_NumberFormat::FORMAT_TEXT);
                $xls->getActiveSheet()->setCellValueExplicit($col.$row, $reg, \PHPExcel_Cell_DataType::TYPE_STRING2);
                $col++;
            }
            $row++;
        }
        
        //Escrever as informações em Temp
        $file = \PHPExcel_IOFactory::createWriter($xls, 'Excel5');
        
        //Onde será salvo o arquivo
        $file->save($exit_path."\/{$this->file_name}.xls");
    }
    
    /**
     * Exibe na tela as informações para o cliente
     */
    public function showInfo()
    {

        echo "<table border=1 cellpadding=0 cellspacing=0>
                 <tr>";

        foreach ($this->objects[0] as $key => $value) {
            echo "<td>$key</td>";
        }
        echo "</tr>";
        foreach ($this->objects as $key => $values) {
            echo "<tr>";
            foreach ($values as $value) {
                echo "<td>$value</td>";
            }
            echo '</tr>';
        }
        
        echo "</table>";        
    }

    public function returnInfo()
    {
        return $this->objects;
    }
    
    /**
     * 
     * Cria um arquivo 
     */
    abstract function createInfo($info);
    abstract function setColumns();
   
    
}
?>
